#include "ElliptProject.h"
#include "AdaptStationary.h"
/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return exp(-10.0 * (x * x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:

  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dow = x.getSize();
    double r2 = (x * x);
    double ux = exp(-10.0 * r2);
    return -(400.0 * r2 - 20.0 * dow) * ux;
  }
};

Elliptdemo::Elliptdemo():
  ellipt("ellipt"),
  adaptInfo(NULL),
  adapt(NULL),
  matrixOperator(NULL),
  rhsOperator(NULL)
{}

Elliptdemo::~Elliptdemo() {
  if(matrixOperator != NULL)
    delete matrixOperator;
  if(rhsOperator != NULL)
    delete rhsOperator;
  if(adapt != NULL)
    delete adapt;
  if(adaptInfo != NULL)
    delete adaptInfo;
}

void Elliptdemo::create(std::string& filename) {
    // ===== init parameters =====
  Parameters::init(true, filename);


  // ===== create and init the scalar problem ===== 
  ellipt.initialize(INIT_ALL);


  // === create adapt info ===
  adaptInfo = new AdaptInfo("ellipt->adapt");


  // === create adapt ===
  adapt = new AdaptStationary("ellipt->adapt", &ellipt, adaptInfo);

  
  // ===== create matrix operator =====
  matrixOperator = new Operator(ellipt.getFeSpace());
  matrixOperator->addSecondOrderTerm(new Laplace_SOT);
  ellipt.addMatrixOperator(matrixOperator);


  // ===== create rhs operator =====
  int degree = ellipt.getFeSpace()->getBasisFcts()->getDegree();
  rhsOperator = new Operator(ellipt.getFeSpace());
  rhsOperator->addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  ellipt.addVectorOperator(rhsOperator);


  // ===== add boundary conditions =====
  ellipt.addDirichletBC(1, new G);


}

int Elliptdemo::solve(SolutionInformation& info) {
  int retCode = adapt->adapt();
  info.dofVec = ellipt.getSolution();
  return retCode;
}

