#include "ParametricProject.h"
#include "AdaptStationary.h"
/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return -2.0 * x[0];
  }
};

Parametricdemo::Parametricdemo():
  parametric("parametric"),
  adaptInfo(NULL),
  adapt(NULL),
  matrixOperator(NULL),
  rhsOperator(NULL)
{}

Parametricdemo::~Parametricdemo() {
   if(matrixOperator != NULL)
    delete matrixOperator;
  if(rhsOperator != NULL)
    delete rhsOperator;
  if(adapt != NULL)
    delete adapt;
  if(adaptInfo != NULL)
    delete adaptInfo;
}

void Parametricdemo::create(string& filename) {
  // ===== init parameters =====
  Parameters::init(false, filename);

  // ===== create and init the scalar problem ===== 
  parametric.initialize(INIT_ALL);

  // === create adapt info ===
  adaptInfo = new AdaptInfo("parametric->adapt", 1);

  // === create adapt ===
  adapt = new AdaptStationary("parametric->adapt",
			       &parametric,
			       adaptInfo);
  
  // ===== create matrix operator =====
  matrixOperator = new Operator(parametric.getFeSpace());
  matrixOperator->addSecondOrderTerm(new Laplace_SOT);
  parametric.addMatrixOperator(matrixOperator);

  // ===== create rhs operator =====
  rhsOperator = new Operator( parametric.getFeSpace());

  int degree = parametric.getFeSpace()->getBasisFcts()->getDegree();

  rhsOperator->addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  parametric.addVectorOperator(rhsOperator);

}

int Parametricdemo::solve(SolutionInformation& info) {
  int retCode = adapt->adapt();
  info.dofVec = parametric.getSolution();
  return retCode;
}
